import unittest
import os

from cv_aid import Frame
import numpy as np
import cv2

base_path = os.path.dirname(__file__)


class Test(unittest.TestCase):
    def test_frame_constructor(self):
        frame = Frame.load(f"{base_path}/test.webp")
        self.assertEqual(frame.width, 400)
        self.assertEqual(frame.height, 600)

    def test_frame_save(self):
        frame = Frame.load(f"{base_path}/test.webp")
        frame.save("test_save.webp",base_path)
        self.assertTrue(os.path.exists(f"{base_path}/test_save.webp"))
        os.remove(f"{base_path}/test_save.webp")

    def test_frame_resize(self):
        frame = Frame.load(f"{base_path}/test.webp")
        frame = frame.resize(width=None, height=100)
        self.assertEqual(frame.width, 400)
        self.assertEqual(frame.height, 100)

    def test_frame_rotate_same_dim(self):
        frame = Frame.load(f"{base_path}/test.webp")
        frame = frame.rotate(90)
        rotated = Frame.load(f"{base_path}/test_rotated.webp")
        self.assertEqual(np.array_equal(frame.frame, rotated.frame), True)

    def test_frame_rotate_different_dim(self):
        frame = Frame.load(f"{base_path}/test.webp")
        frame = frame.rotate(0, same_dim=False)
        self.assertEqual(frame.width, 600)
        self.assertEqual(frame.height, 400)

    def test_frame_crop(self):
        frame = Frame.load(f"{base_path}/test.webp")
        frame = frame.crop(x=0, y=0, width=100, height=100)
        self.assertEqual(frame.width, 100)
        self.assertEqual(frame.height, 100)

    def test_frame_gray(self):
        frame = Frame.load(f"{base_path}/test.webp")
        frame = frame.gray()
        self.assertEqual(frame.frame.shape, (600, 400))

    def test_frame_blur(self):
        frame = Frame.load(f"{base_path}/test.webp")
        frame = frame.blur((5, 5))
        variance = cv2.Laplacian(frame.frame, cv2.CV_64F).var()
        self.assertLess(variance, 100.0)

    def test_frame_canny(self):
        frame = Frame.load(f"{base_path}/test.webp")
        frame = frame.canny(100, 200)
        self.assertEqual(frame.frame.shape, (600, 400))

    def test_frame_line(self):
        frame = Frame.load(f"{base_path}/test.webp")
        frame = frame.line((0, 0), (100, 100), thickness=5)
        lined = Frame.load(f"{base_path}/test_line.webp")
        self.assertEqual(np.array_equal(frame.frame, lined.frame), True)



# pylint: disable=C0103

__docformat__ = "restructuredtext en"

import numpy as np
import dlib


class Dlib:
    """Provides a class for loading and using dlib models."""

    def __init__(self, landmark_predictor_path="shape_predictor_68_face_landmarks.dat"):
        """Initializes Dlib.

        :param face_shape_path: path to face shape predictor
        :type face_shape_path: str
        """

        self.face_detector = dlib.get_frontal_face_detector()
        """Face detector."""

        self.landmark_predictor = dlib.shape_predictor(landmark_predictor_path)
        """Face landmark predictor."""

    def detect_faces(self, image: np.ndarray, **kwargs):
        """Detects faces in an image.

        Args:
          image: image to detect faces in
          kwargs: kwargs for detectMultiScale
          image: np.ndarray:
          **kwargs:

        Returns:
          list of rectangles containing faces

        """

        # Get faces and scores
        faces_ = self.face_detector(image, **kwargs)
        return faces_

    def detect_landmarks(self, image: np.ndarray, rectangle: tuple, **kwargs):
        """Detects landmarks in an image.

        Args:
          image: image to detect landmarks in
          kwargs: kwargs for detectMultiScale
          image: np.ndarray:
          rectangle: tuple:
          **kwargs:

        Returns:
          list of rectangles containing landmarks

        """

        # Get landmarks and scores
        landmarks_ = self.landmark_predictor(image, rectangle, **kwargs)
        return landmarks_

    @classmethod
    def convert_and_trim_bb(cls, image, rectangle):
        """Converts a bounding box to a normal box with respect to the image.

        Args:
          image: image to convert and trim
          rect: bounding box to convert and trim
          rectangle:

        Returns:
          trimmed image

        """
        # extract the starting and ending (x, y)-coordinates of the
        # bounding box
        startX = rectangle.left()
        startY = rectangle.top()
        endX = rectangle.right()
        endY = rectangle.bottom()
        # ensure the bounding box coordinates fall within the spatial
        # dimensions of the image
        startX = max(0, startX)
        startY = max(0, startY)
        endX = min(endX, image.shape[1])
        endY = min(endY, image.shape[0])
        # compute the width and height of the bounding box
        w = endX - startX
        h = endY - startY
        # return our bounding box coordinates
        return (int(startX), int(startY), int(w), int(h))

    @classmethod
    def download_shape_predictor(cls, path="./"):
        """Downloads shape predictor.

        Args:
          path(str, optional): path to download to (Default value = "./")

        Returns:

        """
        return cls.download_and_extract(
            url="http://dlib.net/files/shape_predictor_5_face_landmarks.dat.bz2",
            path=path,
        )

    @classmethod
    def download_face_recognition_model_v1(cls, path="./"):
        """Downloads face recognition model v1.

        Args:
          path(str, optional): path to download to (Default value = "./")

        Returns:

        """
        return cls.download_and_extract(
            url="http://dlib.net/files/dlib_face_recognition_resnet_model_v1.dat.bz2",
            path=path,
        )

    @classmethod
    def download_landmark_detector(cls, path="./"):
        """Downloads landmark detector.

        Args:
          path(str, optional): path to download to (Default value = "./")

        Returns:

        """
        return cls.download_and_extract(
            url="http://dlib.net/files/shape_predictor_68_face_landmarks.dat.bz2",
            path=path,
        )

    @classmethod
    def download_and_extract(cls, url, path="./"):
        """Downloads and extracts shape predictor.

        Args:
          url: url to download file from
          path: path to extract file to (Default value = "./")

        Returns:
          path to downloaded file

        """
        from tqdm import tqdm
        import bz2
        import os
        import requests

        r = requests.get(url, stream=True)
        # Get file name from url
        file_name = r.url.split("/")[-1]
        out_file = os.path.join(path, file_name.split(".")[0] + ".dat")
        file_size = int(r.headers["Content-Length"])
        with open(file_name, "wb") as file, tqdm(
            desc=f"Downloading '{file_name}'",
            total=file_size,
            unit="iB",
            unit_scale=True,
            unit_divisor=1024,
        ) as bar:
            for data in r.iter_content(chunk_size=1024):
                size = file.write(data)
                bar.update(size)
        # Extract file
        with bz2.open(file_name, "rb") as f_in, open(out_file, "wb") as f_out:
            for chunk in tqdm(
                f_in,
                total=os.path.getsize(path),
                unit="B",
                unit_scale=True,
                unit_divisor=1024,
                desc=f"Extracting file '{file_name}'",
            ):
                f_out.write(chunk)
        # Remove compressed file
        os.remove(file_name)
        return out_file



# pylint: disable=C0103
__docformat__ = "restructuredtext en"

import base64
import itertools
from pathlib import Path
from typing import Generator, Iterable

import cv2
import filetype
import numpy as np
from deepstack_sdk import ServerConfig


def find_images(path: Path) -> Generator:
    """Finds all images in a directory.

    Args:
      path(Path): path to the directory to search in
      path: Path:

    Returns:
      generator of all images in the directory

    """
    for file in path.iterdir():
        if file.is_dir():
            continue
        if filetype.is_image(file):
            yield file


def find_videos(path: Path) -> Generator:
    """Finds all videos in a directory.

    Args:
      path(Path): path to the directory to search in
      path: Path:

    Returns:
      generator of all videos in the directory

    """
    for file in path.iterdir():
        if file.is_dir():
            continue
        if filetype.is_video(file):
            yield file


def find_fonts(path: Path) -> Generator:
    """Finds all fonts in a directory.

    Args:
      path(Path): path to the directory to search in
      path: Path:

    Returns:
      generator of all fonts in the directory

    """
    for file in path.iterdir():
        if file.is_dir():
            continue
        if filetype.is_font(file):
            yield file


def rotate(img: np.ndarray, angle, center=None, scale=1.0, same_dim=True) -> np.ndarray:
    """Rotates an image.

    Args:
      img(np.ndarray): image to rotate
      angle(int): angle to rotate the image by
      center(tuple, optional): center of the image to rotate around (Default value = None)
      scale(float, optional): scale of the image (Default value = 1.0)
      same_dim(bool, optional): if True, the image will be resized to the same dimensions as the original (Default value = True)
      img: np.ndarray:

    Returns:
      rotated image

    """

    if not same_dim:
        return cv2.rotate(img, angle)
    # get the dimensions of the image
    (height, width) = img.shape[:2]

    # if the center is None, initialize it as the center of the image
    if center is None:
        center = (width / 2, height / 2)

    # the rotation
    rota_matrix = cv2.getRotationMatrix2D(center, angle, scale)
    rotated_img = cv2.warpAffine(img, rota_matrix, (width, height))

    # return the rotated image
    return rotated_img


def resize(img: np.ndarray, width, height, inter=cv2.INTER_AREA) -> np.ndarray:
    """Resizes an image.

    Args:
      img(np.ndarray): image to resize
      width(int): width of the resized image
      height(int): height of the resized image
      inter(inter: int, optional): interpolation method (Default value = cv2.INTER_AREA)
      img: np.ndarray:

    Returns:
      resized image

    """
    if width is None:
        # calculate the ratio of the height and construct the
        # dimensions
        ratio = height / float(height)
        dim = (int(img.shape[1] * ratio), height)
    else:
        # calculate the ratio of the width and construct the
        # dimensions
        ratio = width / float(width)
        dim = (width, int(height * ratio))

    # resize the image
    resized = cv2.resize(img, dim, interpolation=inter)

    return resized


def concatenate(image1: np.ndarray, image2: np.ndarray, axis=1) -> np.ndarray:
    """Concatenates two images.

    Args:
      image1(np.ndarray): first image to concatenate
      image2(np.ndarray): second image to concatenate
      axis(int, optional): axis to concatenate the images on (Default value = 1)
      image1: np.ndarray:
      image2: np.ndarray:

    Returns:
      concatenated image

    """
    return np.concatenate((image1, image2), axis=axis)


def batch(iterable: Iterable, length: int) -> Generator:
    """Batches an iterable.

    Args:
      iterable(Iterable): iterable to batch
      n(int): number of items to batch
      iterable: Iterable:
      length: int:

    Returns:
      generator of batches

    """
    iterator = iter(iterable)
    while item := list(itertools.islice(iterator, length)):
        yield item


def is_type(obj, type_name):
    """Checks if an object is of a certain type.

    Args:
      obj(object): object to check
      type_name(str): name of the type to check

    Returns:
      True if the object is of the type, False otherwise

    """
    return type(obj).__name__ == type_name


def verify_frame_type(func):
    """Verifies that the frame type is correct.

    Args:
      func(function): function to decorate

    Returns:
      decorated function

    """

    def wrapper(self, frame, *args, **kwargs):
        """

        Args:
          frame:
          *args:
          **kwargs:

        Returns:

        """
        if is_type(frame, "Frame"):
            raise TypeError("The frame must be a Frame object")
        return func(self, frame, *args, **kwargs)

    return wrapper


def copy_frame(func):
    """Copies the frame before applying the function.

    Args:
      func(function): function to decorate

    Returns:
      decorated function

    """

    def wrapper(*args, **kwargs):
        """

        Args:
          *args:
          **kwargs:

        Returns:

        """
        frame = args[0]
        new_frame = type(frame)(frame.frame.copy())
        return func(new_frame, *args[1:], **kwargs)

    return wrapper


def verify_deepstack_config(func):
    """Verifies that the DeepStack config is correct.

    Args:
      func(function): function to decorate

    Returns:
      decorated function

    """

    def wrapper(self, *args, **kwargs):
        """

        Args:
          *args:
          **kwargs:

        Returns:

        """
        server_config = kwargs.get("config")
        if server_config is None:
            try:
                server_config = args[1]
            except KeyError:
                raise ValueError(
                    "The server_config must be a ServerConfig object"
                ) from None

        if server_config is None or not isinstance(server_config, ServerConfig):
            raise ValueError("The server_config must be a ServerConfig object")
        if self.server_config.server_url is None:
            raise ValueError("The server_config must have a server_url")

        return func(self, *args, **kwargs)

    return wrapper


class TemplateResponse:
    """TemplateResponse class."""

    def __init__(self, frame, loc, template):
        """
        Initializes the TemplateResponse class.

        :param frame: frame to apply the template to
        :type frame: np.ndarray
        :param loc: location of the template
        :type loc: tuple
        :param template: template to apply
        :type template: np.ndarray
        """
        self.frame = frame
        self.loc = loc
        self.template = template
        self.width = template.shape[1]
        self.height = template.shape[0]
        self.orig = frame.copy()

    def draw_boxes(self) -> "TemplateResponse":
        """Draws the boxes on the frame.

        :return: The resulting TemplateResponse object

        Args:

        Returns:

        """
        for x, y, width, height, color in self.boxes():
            self.frame = box(self.frame, x, y, width, height, color)
        return self

    def __len__(self):
        return len(self.boxes())

    def __repr__(self):
        return f"TemplateResponse({self.frame})"

    def boxes(self, color=(0, 255, 0)) -> Generator:
        """Returns the boxes of the template.

        Args:
          color(tuple, optional): color of the boxes (Default value = (0)
          255:
          0):

        Returns:
          generator of boxes

        """
        # for x, y in self.loc:
        # yield x, y, self.w, self.h, color
        for point in zip(*self.loc[::-1]):
            yield [
                point[0],
                point[1],
                point[0] + self.width,
                point[1] + self.height,
                color,
            ]


# OpenCV functions


def gray(frame: np.ndarray) -> np.ndarray:
    """Converts a color image to grayscale.

    Args:
      frame(np.ndarray): image to convert to grayscale
      frame: np.ndarray:

    Returns:
      grayscale image

    """
    return cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)


def crop(frame: np.ndarray, x, y, width, height) -> np.ndarray:
    """Crops an image.

    Args:
      frame(np.ndarray): image to crop
      x(int): x coordinate of the top left corner
      y(int): y coordinate of the top left corner
      w(int): width of the crop
      h(int): height of the crop
      frame: np.ndarray:
      width:
      height:

    Returns:
      cropped image

    """
    return frame[y : y + height, x : x + width]


def blur(frame: np.ndarray, ksize=(5, 5)) -> np.ndarray:
    """Blurs an image.

    Args:
      frame(np.ndarray): image to blur
      ksize(tuple, optional): size of the kernel (Default value = (5)
      frame: np.ndarray:
      5):

    Returns:
      blurred image

    """
    return cv2.blur(frame, ksize=ksize)


def flip(frame: np.ndarray, flip_code=1) -> np.ndarray:
    """Flips an image.

    Args:
      frame(np.ndarray): image to flip
      flip_code(int, optional): code for flipping the image (Default value = 1)
      frame: np.ndarray:

    Returns:
      flipped image

    """
    return cv2.flip(frame, flip_code)


def line(
    frame: np.ndarray,
    start: tuple,
    end: tuple,
    color=(0, 255, 0),
    thickness=2,
    line_type=cv2.LINE_8,
) -> np.ndarray:
    """Draws a line on an image.

    Args:
      frame(np.ndarray): image to draw the line on
      start(tuple): start point of the line
      end(tuple): end point of the line
      color(tuple, optional): color of the line (Default value = (0)
      thickness(int, optional): thickness of the line (Default value = 2)
      line_type(int, optional): type of the line (Default value = cv2.LINE_8)
      frame: np.ndarray:
      start: tuple:
      end: tuple:
      255:
      0):

    Returns:
      image with the line

    """
    return cv2.line(frame, start, end, color, thickness, line_type)


def lines(frame, points: list, **kwargs):
    """Draws multiple lines on an image.

    Args:
      frame(np.ndarray): image to draw the lines on
      points(list): list of points to draw lines between
      kwargs(dict): keyword arguments for line
      points: list:
      **kwargs:

    Returns:
      image with the lines

    """
    for i in range(len(points) - 1):
        frame = line(frame, points[i][0], points[i][1], **kwargs)
    return frame


def circle(
    frame: np.ndarray,
    center: tuple,
    radius: int,
    color=(0, 255, 0),
    thickness=2,
    lineType=cv2.LINE_8,
) -> np.ndarray:
    """Draws a circle on an image.

    Args:
      frame(np.ndarray): image to draw the circle on
      center(tuple): center of the circle
      radius(int): radius of the circle
      color(tuple, optional): color of the circle (Default value = (0)
      thickness(int, optional): thickness of the circle (Default value = 2)
      lineType(int, optional): type of the circle (Default value = cv2.LINE_8)

    Returns:
      image with the circle

    """
    return cv2.circle(frame, center, radius, color, thickness, lineType)


def bordered_circle(
    frame: np.ndarray,
    center: np.ndarray,
    radius: np.ndarray,
    color: tuple,
    thickness: int,
    lineType: int,
    border_thickness: int,
    border_color: tuple,
) -> np.ndarray:
    """Draws a circle on an image with a border.

    Args:
      frame(np.ndarray): image to draw the circle on
      center(np.ndarray): center of the circle
      radius(np.ndarray): radius of the circle
      color(tuple, optional): color of the circle (Default value = (0)
      thickness(int, optional): thickness of the circle (Default value = 2)
      lineType(int, optional): type of the circle (Default value = cv2.LINE_8)
      border_thickness(int, optional): thickness of the border (Default value = 2)
      border_color(tuple, optional): color of the border

    Returns:
      image with the circle

    """
    # Draw the circle
    frame = circle(frame, center, radius, color, thickness, lineType)
    # Draw the border
    frame = circle(frame, center, radius, border_color, border_thickness, lineType)
    return frame


def box(
    frame: np.ndarray,
    x,
    y,
    width,
    height,
    color=(0, 255, 0),
    thickness=1,
    line_type=cv2.LINE_8,
    is_max=False,  # pylint: disable=redefined-outer-name
) -> np.ndarray:
    """Draws a box on an image.

    Args:
      frame(np.ndarray): image to draw the box on
      x(int): x coordinate of the top left corner
      y(int): y coordinate of the top left corner
      w(int): width of the box
      h(int): height of the box
      color(tuple, optional): color of the box (Default value = (0)
      thickness(int, optional): thickness of the box (Default value = 1)
      line_type(int, optional): type of the box (Default value = cv2.LINE_8)
      max(bool): if True, treat the box as a max box
      frame: np.ndarray:
      width:
      height:
      255:
      0):
      is_max:  (Default value = False)
      # pylint: disable:  (Default value = redefined-outer-name)

    Returns:
      image with the box

    """
    if is_max:
        frame = cv2.rectangle(
            frame, (x, y), (width, height), color, thickness, line_type
        )
    else:
        frame = cv2.rectangle(
            frame, (x, y), (x + width, y + height), color, thickness, line_type
        )
    return frame


def bordered_box(
    frame: np.ndarray,
    x,
    y,
    width,
    height,
    color: tuple,
    thickness: int,
    line_type: int,
    border_thickness: int,
    border_color: tuple,
) -> np.ndarray:
    """Draws a box on an image with a border.

    Args:
      frame(np.ndarray): image to draw the box on
      x(int): x coordinate of the top left corner
      y(int): y coordinate of the top left corner
      w(int): width of the box
      h(int): height of the box
      color(tuple, optional): color of the box (Default value = (0)
      thickness(int, optional): thickness of the box (Default value = 1)
      line_type(int, optional): type of the box (Default value = cv2.LINE_8)
      border_thickness(int, optional): thickness of the border (Default value = 2)
      border_color(tuple, optional): color of the border

    Returns:
      image with the box

    """
    # Draw the border
    frame = box(
        frame,
        x,
        y,
        width,
        height,
        border_color,
        border_thickness,
        line_type,
    )
    # Draw the box
    frame = box(frame, x, y, width, height, color, thickness, line_type)

    return frame


def boxes(frame, cords, **kwargs):
    """Draws multiple boxes on an image.

    Args:
      frame(np.ndarray): image to draw the boxes on
      cords(list): list of coordinates of the boxes
      kwargs(dict): keyword arguments for box
      **kwargs:

    Returns:
      image with the boxes

    """
    for _box in cords:
        frame = box(frame, *_box, **kwargs)
    return frame


def canny(frame: np.ndarray, threshold1=100, threshold2=200) -> np.ndarray:
    """Applies Canny edge detection to an image.

    Args:
      frame(np.ndarray): image to apply Canny edge detection to
      threshold1(int, optional): first threshold for Canny edge detection (Default value = 100)
      threshold2(int, optional): second threshold for Canny edge detection (Default value = 200)
      frame: np.ndarray:

    Returns:
      image with the Canny edge detection applied

    """
    return cv2.Canny(frame, threshold1, threshold2)


def text(
    frame: np.ndarray,
    text_: str,
    x: int,
    y: int,
    font=cv2.FONT_HERSHEY_SIMPLEX,
    scale=0.5,
    color=(0, 255, 0),
    thickness=2,
) -> np.ndarray:
    """Draws text on an image.

    Args:
      frame(np.ndarray): image to draw the text on
      text_(str): text to draw
      x(int): x coordinate of the top left corner
      y(int): y coordinate of the top left corner
      font(int, optional): font to use for the text (Default value = cv2.FONT_HERSHEY_SIMPLEX)
      scale(float, optional): scale of the text (Default value = 0.5)
      color(tuple, optional): color of the text (Default value = (0)
      thickness(int, optional): thickness of the text (Default value = 2)
      frame: np.ndarray:
      text_: str:
      x: int:
      y: int:
      255:
      0):

    Returns:
      image with the text

    """
    return cv2.putText(frame, text_, (x, y), font, scale, color, thickness, cv2.LINE_AA)


def text_above_box(
    frame: np.ndarray,
    text_: str,
    cords: tuple,
    font=cv2.FONT_HERSHEY_SIMPLEX,
    scale=0.5,
    color=(0, 255, 0),
    thickness=2,
) -> np.ndarray:
    """Draws text above a box on an image.

    Args:
      frame(np.ndarray): image to draw the text on
      text_(str): text to draw
      cords(tuple): coordinates of the box
      font(int, optional): font to use for the text (Default value = cv2.FONT_HERSHEY_SIMPLEX)
      scale(float, optional): scale of the text (Default value = 0.5)
      color(tuple, optional): color of the text (Default value = (0)
      thickness(int, optional): thickness of the text (Default value = 2)
      frame: np.ndarray:
      text_: str:
      cords: tuple:
      255:
      0):

    Returns:
      image with the text

    """
    return text(
        frame,
        text_,
        cords[0],
        cords[1] - int(scale * 30),
        font,
        scale,
        color,
        thickness,
    )


def search(frame, template, method=cv2.TM_CCOEFF_NORMED, threshold=0.8):
    """Searches for a template in an image.

    Args:
      frame(np.ndarray): image to search for the template in
      template(np.ndarray): template to search for
      method(int, optional): method to use for template matching (Default value = cv2.TM_CCOEFF_NORMED)
      threshold(float, optional): threshold for template matching (Default value = 0.8)

    Returns:
      coordinates of the template in the image

    """
    if is_type(template, "Frame"):
        template = template.frame
    result = cv2.matchTemplate(frame, template, method)
    loc = np.where(result >= threshold)
    return TemplateResponse(frame, loc, template)


def stack(frames: list, resize_=None, cols=2) -> np.ndarray:
    """Stacks frames into a single image.

    Args:
      frames(list): frames to stack
      resize_(tuple, optional): resize the frames (Default value = None)
      cols(int, optional): number of columns in the stacked image (Default value = 2)
      frames: list:

    Returns:
      stacked frames

    """
    min_width = min([frame.shape[1] for frame in frames])
    min_height = min([frame.shape[0] for frame in frames])

    if resize_ is not None:
        min_width = resize_[0]
        min_height = resize_[1]
    # Resize each frame to the minimum width and height
    frames = [resize(frame, min_width, min_height) for frame in frames]

    # Calculate the width and height of the stack by summing the width and height of each frame
    _width = sum([frame.shape[1] for frame in frames])
    _height = sum([frame.shape[0] for frame in frames])
    if cols is not None:
        _width = min_width * cols
        rows_count = int((len(frames) / cols))
        if len(frames) % cols != 0:
            rows_count += 1
        _height = rows_count * min_height

    stacked = np.zeros((_height, _width, 3), np.uint8)

    frames_as_rows = batch(frames, cols)
    for row_index, row in enumerate(frames_as_rows):
        col_index = 0
        for col_index, frame in enumerate(row):
            x = col_index * min_width
            y = row_index * min_height
            stacked[y : y + frame.shape[0], x : x + frame.shape[1]] = frame
        if len(row) < cols:
            # Fill the remaining space with white
            x = (col_index * min_width) + (cols - len(row)) * min_width
            y = row_index * min_height

            stacked[
                y : y + min_height,
                x - (min_width * ((cols - len(row)) - 1)) : x + min_width,
            ] = 255

    return stacked


def to_bytes(frame: np.ndarray) -> bytes:
    """Converts a frame to bytes.

    Args:
      frame(np.ndarray): frame to convert
      frame: np.ndarray:

    Returns:
      bytes

    """
    return frame.tobytes()


def to_frame(bytes_: bytes) -> np.ndarray:
    """Converts bytes to a frame.

    Args:
      bytes_(bytes_: bytes): bytes to convert
      bytes_: bytes:

    Returns:
      frame

    """
    return np.frombuffer(bytes_, dtype=np.uint8)


def to_base64(bytes_: bytes) -> str:
    """Converts bytes to a base64 string.

    Args:
      bytes_(bytes_: bytes): bytes to convert
      bytes_: bytes:

    Returns:
      base64 string

    """
    return base64.b64encode(bytes_).decode("utf-8")


def from_base64(base64_: str) -> np.ndarray:
    """Converts a base64 string to bytes.

    Args:
      base64_(str): base64 string to convert
      base64_: str:

    Returns:
      np.ndarray frame

    """
    return to_frame(base64.b64decode(base64_))


def cornerBox(
    frame: np.ndarray,
    box_: tuple,
    corner_length: float,
    corner_thickness: float,
    corner_color: tuple,
    **kwargs,
) -> np.ndarray:
    """Draws a box with corners.

    Args:
      frame(np.ndarray): image to draw the box on
      box(tuple): coordinates of the box
      corner_length(float): length of the corners
      corner_thickness(float): thickness of the corners
      corner_color(tuple): color of the corners
      kwargs: keyword arguments for the box

    Returns:
      image with the box

    """
    x, y, w, h = box_
    if corner_length is None:
        corner_length = min(w, h)
    if corner_thickness is None:
        corner_thickness = corner_length / 10
    if corner_color is None:
        corner_color = (0, 0, 0)
    # Draw the box
    frame = box(frame, *box_, **kwargs)

    # Draw the corners
    # Top left corner
    frame = line(
        frame,
        (x, y),
        (x + corner_length, y),
        color=corner_color,
        thickness=corner_thickness,
    )
    frame = line(
        frame,
        (x, y),
        (x, y + corner_length),
        color=corner_color,
        thickness=corner_thickness,
    )
    # Top right corner
    frame = line(
        frame,
        (x + w - corner_length, y),
        (x + w, y),
        color=corner_color,
        thickness=corner_thickness,
    )
    frame = line(
        frame,
        (x + w, y),
        (x + w, y + corner_length),
        color=corner_color,
        thickness=corner_thickness,
    )
    # Bottom left corner
    frame = line(
        frame,
        (x, y + h - corner_length),
        (x + corner_length, y + h),
        color=corner_color,
        thickness=corner_thickness,
    )
    frame = line(
        frame,
        (x, y + h),
        (x, y + h - corner_length),
        color=corner_color,
        thickness=corner_thickness,
    )
    # Bottom right corner
    frame = line(
        frame,
        (x + w - corner_length, y + h),
        (x + w, y + h),
        color=corner_color,
        thickness=corner_thickness,
    )
    frame = line(
        frame,
        (x + w, y + h),
        (x + w - corner_length, y + h - corner_length),
        color=corner_color,
        thickness=corner_thickness,
    )
    return frame


def dropshade(
    frame: np.ndarray,
    box_: tuple,
    intensity: float,
    shade_color: tuple,
    **kwargs,
) -> np.ndarray:
    """Shades a box.

    Args:
      frame(np.ndarray): image to draw the box on
      box(tuple): coordinates of the box
      intensity(float): intensity of the shading
      color(tuple): color of the shading
      kwargs: keyword arguments for the box

    Returns:
      image with the box

    """
    x, y, w, h = box_
    # Draw the box
    frame = box(frame, *box_, **kwargs)
    # Shade the box
    frame[y : y + h + 1, x : x + w + 1] = (
        frame[y : y + h + 1, x : x + w + 1] * (1 - intensity)
        + np.array(shade_color) * intensity
    )

    return frame


def disk(center: tuple, radius: float, shape: tuple = None) -> list:
    """Generate coordinates of pixels within circle.

    Args:
      center(tuple): center of the circle
      radius(float): radius of the circle
      shape(tuple): shape of the image

    Returns:
      list of coordinates

    """
    if shape is None:
        shape = (1, 1)
    x, y = np.ogrid[: shape[0], : shape[1]]
    dist_from_center = np.sqrt((x - center[0]) ** 2 + (y - center[1]) ** 2)
    mask = dist_from_center <= radius
    return np.where(mask)


def set_color(image: np.ndarray, coords: np.ndarray, color: np.ndarray, alpha: np.float = 1.0 ):
  """
  Set pixel color in the image at the given coordinates.
  Coordinates that exceed the shape of the image will be ignored.
  
  Args:
    image: numpy array of shape (H, W, C)
    coords: numpy array of shape (N, 2)
    color: numpy array of shape (C,)
    alpha: float in [0, 1]
  
  Returns:
    numpy array of shape (H, W, C)
  """
  coords = np.round(coords).astype(np.int32)
  coords = coords[coords[:, 0] >= 0, :]
  coords = coords[coords[:, 0] < image.shape[0], :]
  coords = coords[coords[:, 1] >= 0, :]
  coords = coords[coords[:, 1] < image.shape[1], :]
  image[coords[:, 0], coords[:, 1], :] = (
      image[coords[:, 0], coords[:, 1], :] * (1 - alpha) + color * alpha
  )
  return image





from __future__ import print_function  # Only Python 2.x
import subprocess


def execute(cmd):
    popen = subprocess.Popen(cmd, stdout=subprocess.PIPE, universal_newlines=True)
    for stdout_line in iter(popen.stdout.readline, ""):
        yield stdout_line
    popen.stdout.close()
    return_code = popen.wait()
    if return_code:
        raise subprocess.CalledProcessError(return_code, cmd)


def test():
    """
    Run all unittests.
    """
    cmd = ["poetry", "run", "python", "-u", "-m", "unittest", "discover", "tests", "-v"]
    for stdout_line in execute(cmd):
        print(stdout_line, end="")



import os
import sys
import unittest
import numpy as np
import cv2

sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from cv_aid.utils import rotate, resize, concatenate, stack


class TestRotate(unittest.TestCase):
    """Test rotate"""

    def setup(self):
        """Setup"""
        self.img = cv2.imread(os.path.join(os.path.dirname(__file__), "test.webp"))

    def test_rotate(self):
        """Test rotate"""

        self.setup()
        rotated_90 = rotate(self.img, 90)
        rotated_90_loc = cv2.imread(
            os.path.join(os.path.dirname(__file__), "test_rotated.webp")
        )
        self.assertTrue(np.array_equal(rotated_90, rotated_90_loc))
        difference = cv2.subtract(rotated_90, rotated_90_loc)
        self.assertTrue(np.sum(difference) == 0)


class TestResize(unittest.TestCase):
    """Test resize"""

    def setup(self):
        """Setup"""
        self.img = cv2.imread(os.path.join(os.path.dirname(__file__), "test.webp"))

    def test_resize(self):
        """Test resize"""

        self.setup()
        resized = resize(self.img, width=None, height=100)
        self.assertTrue(resized.shape[0] == 100)


class TestConcatenate(unittest.TestCase):
    """Test concatenate"""

    def setup(self):
        """Setup"""
        self.img = cv2.imread(os.path.join(os.path.dirname(__file__), "egypt.webp"))
        self.img2 = cv2.imread(os.path.join(os.path.dirname(__file__), "egypt.webp"))

    def test_concatenate(self):
        """Test concatenate"""

        self.setup()
        concatenated = concatenate(self.img, self.img2)
        img_path = os.path.join(os.path.dirname(__file__), "test_concatenated.webp")
        concatenated_loc = cv2.imread(img_path)
        self.assertTrue(np.array_equal(concatenated, concatenated_loc))
        difference = cv2.subtract(concatenated, concatenated_loc)
        self.assertTrue(np.sum(difference) == 0)


class TestStack(unittest.TestCase):
    """Test stack"""

    def setup(self):
        """Setup"""
        self.img = cv2.imread(os.path.join(os.path.dirname(__file__), "egypt.webp"))
        self.img2 = cv2.imread(os.path.join(os.path.dirname(__file__), "egypt.webp"))

    def test_stack(self):
        """Test stack"""

        self.setup()
        stacked = stack([self.img, self.img2], cols=2)
        stacked_loc = cv2.imread(
            os.path.join(os.path.dirname(__file__), "test_stacked.webp")
        )
        self.assertTrue(np.array_equal(stacked, stacked_loc))
        difference = cv2.subtract(stacked, stacked_loc)
        self.assertTrue(np.sum(difference) == 0)


if __name__ == "__main__":
    unittest.main()



# pylint: disable=C0103

import numpy as np
import mediapipe as mp


class _Mediapipe:
    """Provides a class for loading and using mediapipe models."""

    def __init__(self, torso_size_multiplier=2.5):
        """Initializes Mediapipe."""

        # Multiplier to apply to the torso to get minimal body size.
        self._torso_size_multiplier = torso_size_multiplier
        "Multiplier to apply to the torso to get minimal body size."

        self.face_detector = mp.solutions.face_detection
        """Face detector."""

        self.hands_detector = mp.solutions.hands
        """Hands detector."""

        self.pose_detector = mp.solutions.pose
        """Pose detector."""
        self.pose_landmark_names = [
            "nose",
            "left_eye_inner",
            "left_eye",
            "left_eye_outer",
            "right_eye_inner",
            "right_eye",
            "right_eye_outer",
            "left_ear",
            "right_ear",
            "mouth_left",
            "mouth_right",
            "left_shoulder",
            "right_shoulder",
            "left_elbow",
            "right_elbow",
            "left_wrist",
            "right_wrist",
            "left_pinky_1",
            "right_pinky_1",
            "left_index_1",
            "right_index_1",
            "left_thumb_2",
            "right_thumb_2",
            "left_hip",
            "right_hip",
            "left_knee",
            "right_knee",
            "left_ankle",
            "right_ankle",
            "left_heel",
            "right_heel",
            "left_foot_index",
            "right_foot_index",
        ]

    def detect_faces(self, image: np.ndarray, **kwargs):
        """Detects faces in an image.

        Args:
          image: image to detect faces in
          kwargs: kwargs for process
          image: np.ndarray:
          **kwargs:

        Returns:
          list of rectangles containing faces

        """

        # Get faces and scores
        faces_ = self.face_detector.process(image, **kwargs)
        if faces_.detections:
            return faces_.detections
        return []

    def detect_hands(self, image: np.ndarray, **kwargs):
        """Detects hands in an image.

        Args:
          image: image to detect hands in
          kwargs: kwargs for process
          image: np.ndarray:
          **kwargs:

        Returns:
          list of rectangles containing hands

        """

        # Get hands and scores
        results = self.hands_detector.process(image, **kwargs)
        if results.multi_hand_landmarks:
            return results.multi_hand_landmarks
        return []

    def detect_pose(self, image: np.ndarray, **kwargs):
        """Detects pose in an image.

        Args:
          image: image to detect pose in
          kwargs: kwargs for process
          image: np.ndarray:
          **kwargs:

        Returns:
          list of rectangles containing pose

        """

        # Get pose and scores
        results = self.pose_detector.process(image, **kwargs)
        if results.pose_landmarks:
            return results.pose_landmarks
        return []

    def left_wrist_angle(self, landmarks):
        """Calculates left wrist angle.

        Args:
          landmarks: pose landmarks

        Returns:
          float: wrist angle in degrees

        """

        # Get wrist and elbow.
        wrist = landmarks[self.pose_landmark_names.index("left_wrist")]
        elbow = landmarks[self.pose_landmark_names.index("left_elbow")]

        # Calculate angle.
        angle = np.arctan2(
            self._get_distance(wrist, elbow)[1], self._get_distance(wrist, elbow)[0]
        )
        return angle * 180 / np.pi

    def right_wrist_angle(self, landmarks):
        """Calculates right wrist angle.

        Args:
          landmarks: pose landmarks

        Returns:
          float: wrist angle in degrees

        """

        # Get wrist and elbow.
        wrist = landmarks[self.pose_landmark_names.index("right_wrist")]
        elbow = landmarks[self.pose_landmark_names.index("right_elbow")]

        # Calculate angle.
        angle = np.arctan2(
            self._get_distance(wrist, elbow)[1], self._get_distance(wrist, elbow)[0]
        )
        return angle * 180 / np.pi

    def is_left_hand_open(self, landmarks, threshold=90):
        """Checks if left hand is open.

        Args:
          landmarks: pose landmarks
          threshold: threshold for hand openness

        Returns:
          bool: True if left hand is open

        """

        angle = self.left_wrist_angle(landmarks)
        return angle > threshold

    def is_right_hand_open(self, landmarks, threshold=90):
        """Checks if right hand is open.

        Args:
          landmarks: pose landmarks
          threshold: threshold for hand openness

        Returns:
          bool: True if right hand is open

        """

        angle = self.right_wrist_angle(landmarks)
        return angle > threshold

    def head_angle(self, landmarks):
        """Calculates head angle.

        Args:
          landmarks: pose landmarks

        Returns:
          float: head angle in degrees

        """

        # Get head and neck.
        head = landmarks[self.pose_landmark_names.index("nose")]
        neck = landmarks[self.pose_landmark_names.index("left_shoulder")]

        # Calculate angle.
        angle = np.arctan2(
            self._get_distance(head, neck)[1], self._get_distance(head, neck)[0]
        )
        return angle * 180 / np.pi

    def is_head_down(self, landmarks, threshold=90):
        """Checks if head is down.

        Args:
          landmarks: pose landmarks
          threshold: threshold for head angle

        Returns:
          bool: True if head is down

        """

        angle = self.head_angle(landmarks)
        return angle < threshold

    def is_head_up(self, landmarks, threshold=90):
        """Checks if head is up.

        Args:
          landmarks: pose landmarks
          threshold: threshold for head angle

        Returns:
          bool: True if head is up

        """

        angle = self.head_angle(landmarks)
        return angle > threshold
    

    def _normalize_pose_landmarks(self, landmarks):
        """Normalizes landmarks translation and scale."""
        landmarks = np.copy(landmarks)

        # Normalize translation.
        pose_center = self._get_pose_center(landmarks)
        landmarks -= pose_center

        # Normalize scale.
        pose_size = self._get_pose_size(landmarks, self._torso_size_multiplier)
        landmarks /= pose_size
        # Multiplication by 100 is not required, but makes it eaasier to debug.
        landmarks *= 100

        return landmarks

    def _get_pose_center(self, landmarks):
        """Calculates pose center as point between hips."""
        left_hip = landmarks[self.pose_landmark_names.index("left_hip")]
        right_hip = landmarks[self.pose_landmark_names.index("right_hip")]
        center = (left_hip + right_hip) * 0.5
        return center

    def _get_pose_size(self, landmarks, torso_size_multiplier):
        """Calculates pose size.

        It is the maximum of two values:
        * Torso size multiplied by `torso_size_multiplier`
        * Maximum distance from pose center to any pose landmark
        """
        # This approach uses only 2D landmarks to compute pose size.
        landmarks = landmarks[:, :2]

        # Hips center.
        left_hip = landmarks[self.pose_landmark_names.index("left_hip")]
        right_hip = landmarks[self.pose_landmark_names.index("right_hip")]
        hips = (left_hip + right_hip) * 0.5

        # Shoulders center.
        left_shoulder = landmarks[self.pose_landmark_names.index("left_shoulder")]
        right_shoulder = landmarks[self.pose_landmark_names.index("right_shoulder")]
        shoulders = (left_shoulder + right_shoulder) * 0.5

        # Torso size as the minimum body size.
        torso_size = np.linalg.norm(shoulders - hips)

        # Max dist to pose center.
        pose_center = self._get_pose_center(landmarks)
        max_dist = np.max(np.linalg.norm(landmarks - pose_center, axis=1))

        return max(torso_size * torso_size_multiplier, max_dist)

    def _get_average_by_names(self, landmarks, name_from, name_to):
        lmk_from = landmarks[self.pose_landmark_names.index(name_from)]
        lmk_to = landmarks[self.pose_landmark_names.index(name_to)]
        return (lmk_from + lmk_to) * 0.5

    def _get_distance_by_names(self, landmarks, name_from, name_to):
        lmk_from = landmarks[self.pose_landmark_names.index(name_from)]
        lmk_to = landmarks[self.pose_landmark_names.index(name_to)]
        return self._get_distance(lmk_from, lmk_to)

    def _get_distance(self, lmk_from, lmk_to):
        return lmk_to - lmk_from

    def _get_pose_distance_embedding(self, landmarks):
        """Converts pose landmarks into 3D embedding.

        We use several pairwise 3D distances to form pose embedding. All distances
        include X and Y components with sign. We different types of pairs to cover
        different pose classes. Feel free to remove some or add new.

        Args:
        landmarks - NumPy array with 3D landmarks of shape (N, 3).

        Result:
        Numpy array with pose embedding of shape (M, 3) where `M` is the number of
        pairwise distances.
        """
        embedding = np.array(
            [
                # One joint.
                self._get_distance(
                    self._get_average_by_names(landmarks, "left_hip", "right_hip"),
                    self._get_average_by_names(
                        landmarks, "left_shoulder", "right_shoulder"
                    ),
                ),
                self._get_distance_by_names(landmarks, "left_shoulder", "left_elbow"),
                self._get_distance_by_names(landmarks, "right_shoulder", "right_elbow"),
                self._get_distance_by_names(landmarks, "left_elbow", "left_wrist"),
                self._get_distance_by_names(landmarks, "right_elbow", "right_wrist"),
                self._get_distance_by_names(landmarks, "left_hip", "left_knee"),
                self._get_distance_by_names(landmarks, "right_hip", "right_knee"),
                self._get_distance_by_names(landmarks, "left_knee", "left_ankle"),
                self._get_distance_by_names(landmarks, "right_knee", "right_ankle"),
                # Two joints.
                self._get_distance_by_names(landmarks, "left_shoulder", "left_wrist"),
                self._get_distance_by_names(landmarks, "right_shoulder", "right_wrist"),
                self._get_distance_by_names(landmarks, "left_hip", "left_ankle"),
                self._get_distance_by_names(landmarks, "right_hip", "right_ankle"),
                # Four joints.
                self._get_distance_by_names(landmarks, "left_hip", "left_wrist"),
                self._get_distance_by_names(landmarks, "right_hip", "right_wrist"),
                # Five joints.
                self._get_distance_by_names(landmarks, "left_shoulder", "left_ankle"),
                self._get_distance_by_names(landmarks, "right_shoulder", "right_ankle"),
                self._get_distance_by_names(landmarks, "left_hip", "left_wrist"),
                self._get_distance_by_names(landmarks, "right_hip", "right_wrist"),
                # Cross body.
                self._get_distance_by_names(landmarks, "left_elbow", "right_elbow"),
                self._get_distance_by_names(landmarks, "left_knee", "right_knee"),
                self._get_distance_by_names(landmarks, "left_wrist", "right_wrist"),
                self._get_distance_by_names(landmarks, "left_ankle", "right_ankle"),
                # Body bent direction.
                # self._get_distance(
                #     self._get_average_by_names(landmarks, 'left_wrist', 'left_ankle'),
                #     landmarks[self._landmark_names.index('left_hip')]),
                # self._get_distance(
                #     self._get_average_by_names(landmarks, 'right_wrist', 'right_ankle'),
                #     landmarks[self._landmark_names.index('right_hip')]),
            ]
        )

        return embedding





import os
import sys
import unittest
from pathlib import Path
from typing import Generator

sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from cv_aid.utils import find_fonts, find_images, find_videos


class TestFonts(unittest.TestCase):
    """Test font"""

    def test_find_fonts(self):
        """Test find_fonts"""
        fonts = find_fonts(Path(os.path.dirname(__file__)))
        self.assertTrue(isinstance(fonts, Generator))
        self.assertTrue(len(list(fonts)) == 0)


class TestImages(unittest.TestCase):
    """Test image"""

    def test_find_images(self):
        """Test find_images"""
        images = find_images(Path(os.path.dirname(__file__)))
        self.assertTrue(isinstance(images, Generator))
        self.assertTrue(len(list(images)) == 7)


class TestVideo(unittest.TestCase):
    """Test video"""

    def test_find_videos(self):
        """Test find_videos"""
        videos = find_videos(Path(os.path.dirname(__file__)))
        self.assertTrue(isinstance(videos, Generator))
        self.assertTrue(len(list(videos)) == 0)



# pylint: disable=C0103
__docformat__ = "restructuredtext en"

import cv2
import numpy as np


class Haarcascades:
    """Class for loading and using haarcascades."""

    def __init__(self):
        """Initializes Haarcascades."""
        self.face_cascade = cv2.CascadeClassifier(
            cv2.data.haarcascades + "haarcascade_frontalface_default.xml"
        )
        """Cascade classifier for faces."""

        self.eye_cascade = cv2.CascadeClassifier(
            cv2.data.haarcascades + "haarcascade_eye.xml"
        )
        """Cascade classifier for eyes."""

        self.smile_cascade = cv2.CascadeClassifier(
            cv2.data.haarcascades + "haarcascade_smile.xml"
        )
        """Cascade classifier for smiles."""

        self.upper_body = cv2.CascadeClassifier(
            cv2.data.haarcascades + "haarcascade_upperbody.xml"
        )
        """Cascade classifier for upper body."""

        self.lower_body = cv2.CascadeClassifier(
            cv2.data.haarcascades + "haarcascade_lowerbody.xml"
        )
        """Cascade classifier for lower body."""

        self.full_body = cv2.CascadeClassifier(
            cv2.data.haarcascades + "haarcascade_fullbody.xml"
        )
        """Cascade classifier for full body."""

        self.profile_face = cv2.CascadeClassifier(
            cv2.data.haarcascades + "haarcascade_profileface.xml"
        )
        """Cascade classifier for profile faces."""

    def detect_faces(self, image: np.ndarray, **kwargs):
        """Detects faces in an image.

        Args:
          image: image to detect faces in
          kwargs: kwargs for detectMultiScale
          image: np.ndarray:
          **kwargs:

        Returns:
          list of rectangles containing faces

        """
        # Get faces and scores
        faces_ = self.face_cascade.detectMultiScale(image, **kwargs)
        return faces_

    def detect_eyes(self, image: np.ndarray, **kwargs):
        """Detects eyes in an image.

        Args:
          image: image to detect eyes in
          kwargs: kwargs for detectMultiScale
          image: np.ndarray:
          **kwargs:

        Returns:
          list of rectangles containing eyes

        """
        # Get eyes and scores
        eyes = self.eye_cascade.detectMultiScale(image, **kwargs)
        return eyes

    def detect_smiles(self, image: np.ndarray, **kwargs):
        """Detects smiles in an image.

        Args:
          image: image to detect smiles in
          kwargs: kwargs for detectMultiScale
          image: np.ndarray:
          **kwargs:

        Returns:
          list of rectangles containing smiles

        """
        # Get smiles and scores
        smiles = self.smile_cascade.detectMultiScale(image, **kwargs)
        return smiles

    def detect_upperbody(self, image: np.ndarray, **kwargs):
        """Detects upper body in an image.

        Args:
          image: image to detect upper body in
          kwargs: kwargs for detectMultiScale
          image: np.ndarray:
          **kwargs:

        Returns:
          list of rectangles containing upper body

        """
        # Get upper body and scores
        upper_body = self.upper_body.detectMultiScale(image, **kwargs)
        return upper_body

    def detect_lowerbody(self, image: np.ndarray, **kwargs):
        """Detects lower body in an image.

        Args:
          image: image to detect lower body in
          kwargs: kwargs for detectMultiScale
          image: np.ndarray:
          **kwargs:

        Returns:
          list of rectangles containing lower body

        """
        # Get lower body and scores
        lower_body = self.lower_body.detectMultiScale(image, **kwargs)
        return lower_body

    def detect_fullbody(self, image: np.ndarray, **kwargs):
        """Detects full body in an image.

        Args:
          image: image to detect full body in
          kwargs: kwargs for detectMultiScale
          image: np.ndarray:
          **kwargs:

        Returns:
          list of rectangles containing full body

        """
        # Get full body and scores
        full_body = self.full_body.detectMultiScale(image, **kwargs)
        return full_body

    def detect_profileface(self, image: np.ndarray, **kwargs):
        """Detects profile face in an image.

        Args:
          image: image to detect profile face in
          kwargs: kwargs for detectMultiScale
          image: np.ndarray:
          **kwargs:

        Returns:
          list of rectangles containing profile face

        """
        # Get profile face and scores
        profile_face = self.profile_face.detectMultiScale(image, **kwargs)
        return profile_face

    def detect_all(self, image: np.ndarray, **kwargs):
        """Detects all objects in an image.

        Args:
          image: image to detect objects in
          kwargs: kwargs for detectMultiScale
          image: np.ndarray:
          **kwargs:

        Returns:
          list of rectangles containing objects

        """
        # Get all objects and scores
        faces_ = self.face_cascade.detectMultiScale(image, **kwargs)
        eyes = self.eye_cascade.detectMultiScale(image, **kwargs)
        smiles = self.smile_cascade.detectMultiScale(image, **kwargs)
        upper_body = self.upper_body.detectMultiScale(image, **kwargs)
        lower_body = self.lower_body.detectMultiScale(image, **kwargs)
        full_body = self.full_body.detectMultiScale(image, **kwargs)
        profile_face = self.profile_face.detectMultiScale(image, **kwargs)
        return {
            "faces": faces_,
            "eyes": eyes,
            "smiles": smiles,
            "upper_body": upper_body,
            "lower_body": lower_body,
            "full_body": full_body,
            "profile_face": profile_face,
        }



# pydoctor --project-base-dir=. --make-html --docformat=restructuredtext cv_aid/
# pylint: disable=C0103
__docformat__ = "restructuredtext en"

import os
import cv2
import numpy as np

from cv_aid import utils
from cv_aid.haarcascades import Haarcascades

_is_dl_available = True
# Check if dlib is installed
try:
    import dlib
    from cv_aid._dlib import Dlib
    _dlib = Dlib()
except ImportError:
    _is_dl_available = False



class Frame:  # pylint: disable=too-many-public-methods
    """A class to represent a frame of video."""

    def __init__(self, frame):
        """Initialize a Frame object.

        Args:
          frame(numpy.ndarray): The frame to wrap.
        """
        self.frame = frame
        self._haarcascades = None
        self._dlib = None

    @classmethod
    def load(cls, path) -> "Frame":
        """Load a frame from a file.

        Args:
          path(str): Path to the file.

        Returns:
          The resulting frame.

        """
        return cls(cv2.imread(str(path)))

    @property
    def shape(self) -> tuple:
        """Get the shape of the frame."""
        return self.frame.shape

    @property
    def width(self) -> int:
        """Get the width of the frame."""
        return self.shape[1]

    @property
    def height(self) -> int:
        """Get the height of the frame."""
        return self.shape[0]

    @property
    def size(self) -> int:
        """Get the size of the frame."""
        return self.shape[:2]

    @property
    def channels(self) -> int:
        """Get the number of channels in the frame."""
        return self.shape[2]

    def to_bytes(self) -> bytes:
        """Convert the frame to bytes.

        Returns:
          The resulting bytes.

        """
        return self.frame.tobytes()

    def gray(self) -> "Frame":
        """Convert the frame to grayscale.

        :return: The resulting frame.

        Args:

        Returns:

        """
        return Frame(utils.gray(self.frame))

    def resize(self, width=None, height=None, inter=cv2.INTER_AREA) -> "Frame":
        """Resize the frame.

        Args:
          width(int, optional): The new width. (Default value = None)
          height(int, optional): The new height. (Default value = None)
          inter(inter: inter: int, optional): The interpolation method. (Default value = cv2.INTER_AREA)

        Returns:
          The resulting frame.

        """
        return Frame(utils.resize(self.frame, width, height, inter))

    def rotate(self, angle, center=None, scale=1.0, same_dim=True) -> "Frame":
        """Rotate the frame.

        Args:
          angle(float): The angle to rotate the frame by.
          center(tuple, optional): The center of rotation. (Default value = None)
          scale(float, optional): The scale to apply. (Default value = 1.0)
          same_dim: Default value = True)

        Returns:
          The resulting frame.

        """
        return Frame(utils.rotate(self.frame, angle, center, scale, same_dim))

    def flip(self, flip_code) -> "Frame":
        """Flip the frame.

        Args:
          flip_code(int): The code for the flip.

        Returns:
          The resulting frame.

        """
        return Frame(utils.flip(self.frame, flip_code))

    def crop(self, x, y, width, height) -> "Frame":  # pylint: disable=invalid-name
        """Crop the frame.

        Args:
          x(int): The x coordinate of the top left corner.
          y(int): The y coordinate of the top left corner.
          width(int): The width of the crop.
          height(int): The height of the crop.

        Returns:
          The resulting frame.

        """
        return Frame(utils.crop(self.frame, x, y, width, height))

    def crop_to_size(self, width, height) -> "Frame":
        """Crop the frame to a specific size.

        Args:
          width(int): The width of the frame.
          height(int): The height of the frame.

        Returns:
          The resulting frame.

        """
        return self.crop(0, 0, width, height)

    def crop_to_ratio(self, ratio) -> "Frame":
        """Crop the frame to a specific ratio.

        Args:
          ratio(float): The ratio to crop to.

        Returns:
          The resulting frame.

        """
        return self.crop_to_size(int(self.width / ratio), int(self.height / ratio))

    def crop_to_ratio_width(self, ratio) -> "Frame":
        """Crop the frame to a specific ratio.

        Args:
          ratio(float): The ratio to crop to.

        Returns:
          The resulting frame.

        """
        return self.crop_to_size(int(self.width / ratio), self.height)

    def crop_to_ratio_height(self, ratio) -> "Frame":
        """Crop the frame to a specific ratio.

        Args:
          ratio(float): The ratio to crop to.

        Returns:
          The resulting frame.

        """
        return self.crop_to_size(self.width, int(self.height / ratio))

    def blur(self, ksize=5) -> "Frame":
        """Blur the frame.

        Args:
          ksize(int, optional): The kernel size. (Default value = 5)

        Returns:
          The resulting frame.

        """
        return Frame(utils.blur(self.frame, ksize))

    def canny(self, threshold1, threshold2) -> "Frame":
        """Apply the Canny edge detector.

        Args:
          threshold1(int): The first threshold.
          threshold2(int): The second threshold.

        Returns:
          The resulting frame.

        """
        return Frame(utils.canny(self.frame, threshold1, threshold2))

    def line(
        self, start, end, color=(0, 255, 0), thickness=1, line_type=cv2.LINE_8
    ) -> "Frame":
        """Draw a line on the frame.

        Args:
          start(tuple): The start of the line.
          end(tuple): The end of the line.
          color(tuple, optional): The color of the line. (Default value = (0)
          thickness(int, optional): The thickness of the line. (Default value = 1)
          line_type(int, optional): The type of the line. (Default value = cv2.LINE_8)
          255: param 0):
          0):

        Returns:
          The resulting frame.

        """
        return Frame(utils.line(self.frame, start, end, color, thickness, line_type))

    def circle(
        self, center, radius, color=(0, 255, 0), thickness=1, line_type=cv2.LINE_8
    ) -> "Frame":
        """Draw a circle on the frame.

        Args:
          center(tuple): The center of the circle.
          radius(int): The radius of the circle.
          color(tuple, optional): The color of the circle. (Default value = (0)
          thickness(int, optional): The thickness of the circle. (Default value = 1)
          line_type(int, optional): The type of the circle. (Default value = cv2.LINE_8)

        Returns:
          The resulting frame.

        """
        return Frame(
            utils.circle(self.frame, center, radius, color, thickness, line_type)
        )

    def box(
        self,
        x,
        y,
        width,
        height,
        color,
        thickness=1,
        line_type=cv2.LINE_8,  # pylint: disable=invalid-name
        is_max=False,
    ) -> "Frame":
        """Draw a box on the frame.

        Args:
          x(int): The x coordinate of the top left corner.
          y(int): The y coordinate of the top left corner.
          width(int): The width of the box.
          height(int): The height of the box.
          color(tuple): The color of the box.
          thickness(int, optional): The thickness of the box. (Default value = 1)
          line_type(int, optional): The type of the box. (Default value = cv2.LINE_8)
          is_max(bool, optional): Whether or not to draw the max box. (Default value = False)
          
        Returns:
          The resulting frame.

        """
        return Frame(
            utils.box(
                self.frame,
                x,
                y,
                width,
                height,
                color,
                thickness,
                line_type,
                is_max=is_max,
            )
        )

    def lines(self, points, color, thickness=1, line_type=cv2.LINE_8) -> "Frame":
        """Draw lines on the frame.

        Args:
          points(List[Tuple[int, int]]): The points to draw.
          color(tuple): The color of the lines.
          thickness(int, optional): The thickness of the lines. (Default value = 1)
          line_type(int, optional): The type of the lines. (Default value = cv2.LINE_8)

        Returns:
          The resulting frame.

        """
        return Frame(
            utils.lines(
                self.frame,
                points,
                color=color,
                thickness=thickness,
                line_type=line_type,
            )
        )

    def boxes(
        self, boxes, color, thickness=1, line_type=cv2.LINE_8, is_max=False
    ) -> "Frame":
        """Draw boxes on the frame.

        Args:
          boxes(List[Tuple[int, int, int, int]]): The boxes to draw.
          color(tuple): The color of the boxes.
          thickness(int, optional): The thickness of the boxes. (Default value = 1)
          line_type(int, optional): The type of the boxes. (Default value = cv2.LINE_8)
          is_max: Default value = False)

        Returns:
          The resulting frame.

        """
        return Frame(
            utils.boxes(
                self.frame,
                boxes,
                color=color,
                thickness=thickness,
                line_type=line_type,
                is_max=is_max,
            )
        )

    def text(
        self,
        text,
        position,
        font_face=cv2.FONT_HERSHEY_SIMPLEX,
        font_scale=1.0,
        color=(0, 255, 0),
        thickness=1,
    ) -> "Frame":
        """Draw text on the frame.

        Args:
          text(str): The text to draw.
          position(tuple): The position of the text.
          color(tuple, optional): The color of the text. (Default value = (0)
          font_face(int, optional): The font face of the text. (Default value = cv2.FONT_HERSHEY_SIMPLEX)
          font_scale(float, optional): The font scale of the text. (Default value = 1.0)
          thickness(int, optional): The thickness of the text. (Default value = 1)
          255: param 0):
          0):

        Returns:
          The resulting frame.

        """
        return Frame(
            utils.text(
                self.frame,
                text,
                *position,
                font_face,
                font_scale,
                color,
                thickness,
            )
        )

    def __add__(self, other) -> "Frame":
        """Add two frames together.

        :param other: The other frame to be added.
        :type other: Frame
        :return: The resulting frame.
        """
        return Frame(self.frame + other.frame)

    def __sub__(self, other) -> "Frame":
        """Subtract two frames.

        :param other: The other frame to be subtracted.
        :type other: Frame
        :return: The resulting frame.
        """
        return Frame(self.frame - other.frame)

    def __mul__(self, other) -> "Frame":
        """Multiply two frames.

        :param other: The other frame to be multiplied.
        :type other: Frame
        :return: The resulting frame.
        """
        return Frame(self.frame * other.frame)

    def __truediv__(self, other) -> "Frame":
        """Divide two frames.

        :param other: The other frame to be divided.
        :type other: Frame
        :return: The resulting frame.
        """
        return Frame(self.frame / other.frame)

    def abs(self) -> "Frame":
        """Take the absolute value of the frame.

        :return: The resulting frame.

        Args:

        Returns:

        """
        return Frame(np.abs(self.frame))

    def show(self, title="Frame") -> None:
        """Show the frame.

        Args:
          title(str, optional): The title of the frame. (Default value = "Frame")

        Returns:

        """
        cv2.imshow(title, self.frame)

    def save(self, name, path="./"):
        """Save the frame.

        Args:
          path(str): The path to save the frame.
          name(str): The name of the frame.

        Returns:

        """
        cv2.imwrite(os.path.join(path, name), self.frame)

    def __repr__(self) -> str:
        """Get the string representation of the frame.

        :return: The string representation of the frame.
        """
        return f"Frame({self.frame.shape})"

    @property
    def haarcascades(self):
        """Provides access to the haarcascades."""
        # Create the haarcascades class if it doesn't exist
        if not hasattr(self, "_haarcascades") or getattr(self, "_haarcascades") is None:
            self._haarcascades = Haarcascades()
        return self._haarcascades

    @property
    def dlib(self):
        """Provides access to the dlib."""
        if not _is_dl_available:
          raise ImportError(
              "Dlib is not installed. Please install it with `pip install dlib`. or run `pip install cv_aid[dlib]`."
          )
        # Create the dlib class if it doesn't exist
        if not hasattr(self, "_dlib") or getattr(self, "_dlib") is None:
            self._dlib = _dlib
        return self._dlib



# pylint: disable=C0103
__docformat__ = "restructuredtext en"

from threading import Thread

import cv2

from cv_aid import Frame


class VideoStream:
    """A class for streaming video from a camera.

    Args:
      src: The source of the video stream.

    Returns:

    """

    def __init__(
        self,
        src,
        width=None,
        height=None,
        on_frame=None,
        callback_args=(),
        callback_kwargs={},
    ):
        """
        Initialize the video stream and read the first frame from the stream.

        :param src: The source of the video stream.
        :param width: The width of the frame.
        :type width: int
        :param height: The height of the frame.
        :type height: int
        """
        self.src = src
        self.stopped = False  # Indicates if the video stream is stopped.
        self.paused = False  # Indicates if the video stream is paused.
        self.stream = cv2.VideoCapture(src)  # The video stream.
        if width is not None and height is not None:
            self.stream.set(cv2.CAP_PROP_FRAME_WIDTH, width)
            self.stream.set(cv2.CAP_PROP_FRAME_HEIGHT, height)
        self.on_frame = on_frame
        self.callback_args = callback_args
        self.callback_kwargs = callback_kwargs
        self.frame = self.read()  # The current frame.
        self.thread = Thread(
            target=self.update, args=()
        )  # The thread that reads the video stream.
        # self.thread.daemon = True  # Make the thread a daemon thread.
        self.is_window_open = False  # Indicates if a window is open.

    def read(self) -> Frame:
        """Read a frame from the video stream and return it."""
        success, frame = self.stream.read()
        if not success:
            self.stop()
        frame = Frame(frame)
        if self.on_frame is not None:
            frame = self.on_frame(frame, *self.callback_args, **self.callback_kwargs)
            if not isinstance(frame, Frame):
                frame = Frame(frame)
        return frame

    def start(self):
        """Start the video stream."""
        self.thread.start()
        return self

    def update(self):
        """ """
        while True:
            if self.stopped:
                return
            if self.paused:
                continue
            self.frame = self.read()

    def pause(self):
        """Pause the video stream."""
        self.paused = True

    def resume(self):
        """Resume the video stream."""
        self.paused = False

    def stop(self):
        """Stop the video stream."""
        self.stopped = True
        self.kill()
        return self

    def __del__(self):
        self.kill()

    def __exit__(self, exc_type, exc_value, traceback):
        self.kill()

    def kill(self):
        """Kill the video stream."""
        self.stream.release()
        self.thread.join()
        self.close_windows()

    def start_window(self, title=None):
        """Start a window to display the video stream.

        Args:
          title(str, optional): The title of the window. (Default value = None)

        Returns:

        """
        self.is_window_open = True
        if title is None:
            title = f"Streaming '{self.src}' Frames"
        while True:
            self.read().show(title)
            if cv2.waitKey(1) & 0xFF == ord("q"):
                self.close_windows()
                break

    def close_windows(self):
        """ """
        cv2.destroyAllWindows()
        self.is_window_open = False



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['cv_aid', 'tests']

package_data = \
{'': ['*']}

modules = \
['scripts']
install_requires = \
['cmake>=3.22.1,<4.0.0',
 'deepstack-sdk>=0.2.1,<0.3.0',
 'dlib>=19.22.1,<20.0.0',
 'filetype>=1.0.9,<2.0.0',
 'numpy>=1.21,<2.0',
 'opencv-python>=4.1.0,<5.0.0',
 'requests>=2.27.1,<3.0.0',
 'tqdm>=4.62.3,<5.0.0']

entry_points = \
{'console_scripts': ['test = scripts:test']}

setup_kwargs = {
    'name': 'cv-aid',
    'version': '1.0.1',
    'description': 'CV Aid is a set of helpers for computer vision tasks.',
    'long_description': '# cv-aid\n\nCV Aid is a set of helpers of computer vision tasks.\n\n## Installation\n\n`pip install cv-aid`\n\n### From source\n\n```\ngit clone https://github.com/khalidelboray/cv-aid\ncd cv-aid\npoetry install\npoetry run python setup.py install\n```\n\n## Tests\n\n`poetry run test`\n\nall tests are in `tests/` directory.\n\n## Examples\n\n- Basic Frame Functions\n\n    ```python\n    from cv_aid import Frame\n\n    frame = Frame.load(\'/path/to/image.jpg\')\n    # or\n    import cv2\n    frame = Frame(cv2.imread(\'/path/to/image.jpg\'))\n\n    # Grayscale image\n    gray = frame.gray()\n\n    # Resize image\n    small = frame.resize(width=100, height=100)\n\n    # Crop image\n    cropped = frame.crop(x=100, y=100, width=100, height=100)\n\n    # All methods return a new Frame object, so you can chain them\n    new_frame = frame.resize(width=100, height=100).crop(x=100, y=100, width=100, height=100)\n\n    # Save image\n    frame.save(\'/path/to/image.jpg\')\n    ```\n\n- Basic Video Functions\n\n    ```python\n    from cv_aid import VideoStream, Frame\n    import cv2\n    import numpy as np\n\n\n    def on_frame(frame: Frame) -> Frame:\n        """\n        A function that is called when a frame is read from the video stream.\n\n        :param frame: The frame that was read.\n        :return: The frame that was read.\n        """\n        orig = frame\n        canny = frame.gray().canny(50, 100)\n        line_image = Frame(np.copy(orig.frame) * 0)\n        lines = cv2.HoughLinesP(\n            canny.frame, 1, np.pi / 180, 50, np.array([]), minLineLength=10, maxLineGap=5\n        )\n        if lines is not None:\n            for line in lines:\n                line = line[0]\n                line_image = line_image.line(\n                    (line[0], line[1]), (line[2], line[3]), (0, 255, 0), 3\n                )\n        lines_edges = cv2.addWeighted(orig.frame, 0.8, line_image.frame, 1, 1)\n        return Frame(lines_edges)\n\n\n    stream = VideoStream(src=0, on_frame=on_frame).start()\n    stream.start_window()\n    ```\n\n    *Output Demo:*\n\n    ![Code Window](https://raw.githubusercontent.com/khalidelboray/cv-aid/master/images/stream.png)\n\n- Haar Cascade Functions\n\n    ```python\n    from cv_aid import VideoStream, Frame\n\n    def on_frame(frame: Frame) -> Frame:\n        """\n        A function that is called when a frame is read from the video stream.\n\n        :param frame: The frame that was read.\n        :return: The frame that was read.\n        """\n        boxes = frame.haarcascades.detect_faces(frame.frame, scaleFactor=1.1, minNeighbors=5, minSize=(30, 30))\n        frame = frame.boxes(boxes, color=(0, 255, 0))\n        return frame\n\n\n    if __name__ == "__main__":\n\n        stream = VideoStream(src=0, on_frame=on_frame).start()\n        stream.start_window()\n    ```\n\n    *Output Demo:*\n\n    ![haarcascade Window](https://raw.githubusercontent.com/khalidelboray/cv-aid/master/images/haarcascades.png)\n\n- Tourch Hub (Yolov5)\n\n    ```python\n    from cv_aid import VideoStream, Frame\n    import torch\n\n    model = torch.hub.load(\'ultralytics/yolov5\', \'yolov5s\')\n\n    def on_frame(frame: Frame) -> Frame:\n        """\n        A function that is called when a frame is read from the video stream.\n\n        :param frame: The frame that was read.\n        :return: The frame that was read.\n        """\n        results = model(frame.frame)\n        results.display(render=True)\n        frame = Frame(results.imgs[0])    \n        return frame\n\n\n    if __name__ == "__main__":\n        \n        stream = VideoStream(src=0, on_frame=on_frame).start()\n        stream.start_window()\n    ```\n\n    ![torch yolov5](https://raw.githubusercontent.com/khalidelboray/cv-aid/master/images/torch_yolo.png)\n\n- Dlib (`Download model`)\n\n    ```python\n    from cv_aid._dlib import Dlib\n    saved_model = Dlib.download_landmark_detector(path=\'/dir/to/download/model/at/\')\n    dlib = Dlib(landmark_predictor_path=saved_model)\n\n    face_recognetion_model = Dlib.download_face_recognition_model_v1(path=\'/dir/to/download/model/at/\')\n    ```\n\n- Dlib (Face landmark)\n\n    `Give it a try!`\n\n    ```python\n    # pylint: disable=C0103\n\n    import math\n\n    import cv2\n    import numpy as np\n    from skimage.draw import disk, polygon, set_color\n\n    from cv_aid import Frame, VideoStream\n\n    RIGHT_EYE_POINTS = list(range(36, 42))\n    LEFT_EYE_POINTS = list(range(42, 48))\n\n\n    def get_poly_data(desired, landmarks, shape):\n        points = []\n        for i in desired:\n            points.append((landmarks.part(i).x, landmarks.part(i).y))\n        points = np.array(points, dtype=np.int32)\n        rr, cc = polygon(points[:, 1], points[:, 0], shape)\n        return points, rr, cc\n\n\n    def on_frame(frame: Frame) -> Frame:\n        """\n        A function that is called when a frame is read from the video stream.\n\n        :param frame: The frame that was read.\n        :return: The frame that was read.\n        """\n\n        faces = frame.dlib.detect_faces(frame.frame)\n        for face in faces:\n            face_landmarks = frame.dlib.detect_landmarks(frame.frame, face)\n            left_eye, *_ = get_poly_data(LEFT_EYE_POINTS, face_landmarks, frame.shape)\n            right_eye, *_ = get_poly_data(RIGHT_EYE_POINTS, face_landmarks, frame.shape)\n\n            left_eye_center = left_eye.mean(axis=0).astype("int")\n            right_eye_center = right_eye.mean(axis=0).astype("int")\n            left_eye_radius = (\n                int(\n                    math.sqrt(\n                        (left_eye[3][0] - left_eye[0][0]) ** 2\n                        + (left_eye[3][1] - left_eye[0][1]) ** 2\n                    )\n                )\n                - 10\n            )\n            right_eye_radius = (\n                int(\n                    math.sqrt(\n                        (right_eye[3][0] - right_eye[0][0]) ** 2\n                        + (right_eye[3][1] - right_eye[0][1]) ** 2\n                    )\n                )\n                - 10\n            )\n            frame = (\n                # Glasses connection line\n                frame.line(\n                    (left_eye_center[0] - left_eye_radius, left_eye_center[1]),\n                    (right_eye_center[0] + right_eye_radius, right_eye_center[1]),\n                    (0, 0, 0),\n                    4,\n                )\n                # Glasses circle 1 *Border*\n                .circle(\n                    left_eye_center,\n                    left_eye_radius,\n                    (0, 0, 0),\n                    4,\n                )\n                # Glasses circle 1\n                .circle(\n                    left_eye_center,\n                    left_eye_radius,\n                    (0, 0, 255),\n                    2,\n                )\n                # Glasses circle 2 *Border*\n                .circle(\n                    right_eye_center,\n                    right_eye_radius,\n                    (0, 0, 0),\n                    4,\n                )\n                # Glasses circle 2\n                .circle(\n                    right_eye_center,\n                    right_eye_radius,\n                    (0, 0, 255),\n                    2,\n                )\n                # Ears connection line 1\n                .line(\n                    (face_landmarks.part(0).x, face_landmarks.part(0).y),\n                    (right_eye_center[0] - right_eye_radius, right_eye_center[1]),\n                    (0, 0, 255),\n                    2,\n                )\n                # Ears connection line 1\n                .line(\n                    (face_landmarks.part(16).x, face_landmarks.part(16).y),\n                    (left_eye_center[0] + left_eye_radius, left_eye_center[1]),\n                    (0, 0, 255),\n                    2,\n                )\n            )\n            # Overlay the frame with the image of the glasses colored in transparent black\n            overlay = frame.frame.copy()\n            alpha = 0.5\n            # Get first circle rows and columns (pixel coordinates)\n            rr, cc = disk(right_eye_center[::-1], right_eye_radius)\n            # Set the color of the circle\n            set_color(overlay, (rr, cc), (0, 0, 0))\n\n            # Get second circle rows and columns (pixel coordinates)\n            rr, cc = disk(left_eye_center[::-1], left_eye_radius)\n            # Set the color of the circle\n            set_color(overlay, (rr, cc), (0, 0, 0))\n\n            # Overlay the image with the overlay image\n            frame.frame = cv2.addWeighted(overlay, alpha, frame.frame, 1 - alpha, 0)\n\n        return frame\n\n\n    if __name__ == "__main__":\n\n        stream = VideoStream(src=0, on_frame=on_frame).start()\n        stream.start_window()\n\n    ```\n',
    'author': 'Khalid Mohamed Elborai',
    'author_email': 'accnew820@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://khalidelboray.me/cv-aid',
    'packages': packages,
    'package_data': package_data,
    'py_modules': modules,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
